@@ -1,20 +1,21 @@
 //! A Label Storage Area
 
 mod label_index_block;
-mod namespace_label;
-mod region_label;
+pub mod namespace_label;
+pub mod region_label;
 
 use crate::lsa::{
     label_index_block::LabelIndexBlock,
     namespace_label::*,
     region_label::*,
-    Label::{EMPTY, NAMESPACE, REGION, VENDOR},
+    Label::{Empty, Namespace, Region, Vendor},
 };
 use serde::{
     ser::{SerializeStruct, Serializer},
     Serialize,
 };
 use std::{
+    fmt,
     io::{Error, ErrorKind, Write},
     slice,
 };
@@ -84,7 +85,7 @@ fn generate_checksum(dwords: &[u32]) -> u64 {
     checksum.value()
 }
 
-fn label_from_bytes(raw_label: &[u8]) -> Result<Label, std::io::Error> {
+pub fn label_from_bytes(raw_label: &[u8]) -> Result<Label, std::io::Error> {
     assert!(raw_label.len() == 256);
     let uuid: Uuid = match Uuid::from_slice(&raw_label[..16]) {
         Ok(u) => u,
@@ -93,9 +94,9 @@ fn label_from_bytes(raw_label: &[u8]) -> Result<Label, std::io::Error> {
 
     // As labels are a fixed size, these should all be safe.
     let label: Label = match uuid {
-        REGION_UUID => REGION(RegionLabel::from_bytes(raw_label)),
-        NAMESPACE_UUID => NAMESPACE(NamespaceLabel::from_bytes(raw_label)),
-        _ => EMPTY,
+        REGION_UUID => Region(RegionLabel::from_bytes(raw_label)),
+        NAMESPACE_UUID => Namespace(NamespaceLabel::from_bytes(raw_label)),
+        _ => Empty,
     };
 
     Ok(label)
@@ -127,6 +128,8 @@ pub trait CommonLabelProperties {
     fn get_size(&self) -> usize {
         256
     }
+
+    fn set_slot(&mut self, size: usize);
 }
 
 /// Vendor labels in the Label Storage area.
@@ -134,7 +137,7 @@ pub trait CommonLabelProperties {
 /// Vendor label's data is entirely opaque.
 #[repr(packed)]
 #[derive(Clone, Copy, Serialize)]
-#[allow(non_snake_case,dead_code)]
+#[allow(non_snake_case, dead_code)]
 pub struct VendorLabel {
     #[serde(serialize_with = "to_uuid")]
     Type: [u8; 16],
@@ -157,38 +160,53 @@ impl CommonLabelProperties for VendorLabel {
         assert!(buf.len() == 256);
         unsafe { std::ptr::read(buf.as_ptr() as *const _) }
     }
+    fn set_slot(&mut self, _slot: usize) {
+        panic!("Invalid set_slot");
+    }
 }
 
 /// A Label occupying a slot in the Label Storage Area
+#[derive(Serialize)]
 pub enum Label {
-    REGION(RegionLabel),
-    NAMESPACE(NamespaceLabel),
-    VENDOR(VendorLabel),
-    EMPTY,
+    Region(RegionLabel),
+    Namespace(NamespaceLabel),
+    Vendor(VendorLabel),
+    Empty,
+}
+
+impl fmt::Display for Label {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        match *self {
+            Label::Region(_) => write!(f, "Region Label"),
+            Label::Namespace(_) => write!(f, "Namespace Label"),
+            Label::Vendor(_) => write!(f, "Vendor Label"),
+            Label::Empty => write!(f, "?"),
+        }
+    }
 }
 
 impl CommonLabelProperties for Label {
     fn update_checksum(&mut self) {
         match self {
-            REGION(r) => r.update_checksum(),
-            NAMESPACE(n) => n.update_checksum(),
-            VENDOR(v) => v.update_checksum(),
-            _ => { }
+            Region(r) => r.update_checksum(),
+            Namespace(n) => n.update_checksum(),
+            Vendor(v) => v.update_checksum(),
+            _ => {}
         };
     }
     fn verify(&self) -> Result<(), std::io::Error> {
         match self {
-            REGION(r) => r.verify(),
-            NAMESPACE(n) => n.verify(),
-            VENDOR(v) => v.verify(),
+            Region(r) => r.verify(),
+            Namespace(n) => n.verify(),
+            Vendor(v) => v.verify(),
             _ => Ok(()),
         }
     }
     fn as_bytes(&self) -> &[u8] {
         match self {
-            REGION(r) => r.as_bytes(),
-            NAMESPACE(n) => n.as_bytes(),
-            VENDOR(v) => v.as_bytes(),
+            Region(r) => r.as_bytes(),
+            Namespace(n) => n.as_bytes(),
+            Vendor(v) => v.as_bytes(),
             _ => unreachable!(),
         }
     }
@@ -196,6 +214,14 @@ impl CommonLabelProperties for Label {
     fn from_bytes(buf: &[u8]) -> Self {
         label_from_bytes(buf).expect("Couldn't decipher label")
     }
+
+    fn set_slot(&mut self, slot: usize) {
+        match self {
+            Region(r) => r.set_slot(slot),
+            Namespace(n) => n.set_slot(slot),
+            _ => unreachable!(),
+        }
+    }
 }
 
 fn next_seq(seq: u32) -> u32 {
@@ -357,17 +383,17 @@ impl LabelStorageArea {
                 (2, 0)
             }
             // Only single valid label index block
-            (false, true) => (0 as usize, lib0.get_sequence()),
-            (true, false) => (1 as usize, lib1.get_sequence()),
+            (false, true) => (0_usize, lib0.get_sequence()),
+            (true, false) => (1_usize, lib1.get_sequence()),
             // If both are valid
             (false, false) => {
                 match (lib0.get_sequence(), lib1.get_sequence()) {
                     // Second block wins when same sequence
-                    (1, 1) | (2, 2) | (3, 3) => (1 as usize, lib1.get_sequence()),
+                    (1, 1) | (2, 2) | (3, 3) => (1_usize, lib1.get_sequence()),
                     // Block 0 has a "higher" sequence
-                    (2, 1) | (3, 2) | (1, 3) => (0 as usize, lib0.get_sequence()),
+                    (2, 1) | (3, 2) | (1, 3) => (0_usize, lib0.get_sequence()),
                     // Block 1 has a "higher" sequence
-                    (1, 2) | (2, 3) | (3, 1) => (1 as usize, lib1.get_sequence()),
+                    (1, 2) | (2, 3) | (3, 1) => (1_usize, lib1.get_sequence()),
                     (a, b) => panic!("Unexpected sequence ({}, {})", a, b),
                 }
             }
@@ -408,7 +434,7 @@ impl LabelStorageArea {
     //    }
     //
 
-    fn push_label(&mut self, label: Label) -> Result<usize, std::io::Error> {
+    pub fn push_label(&mut self, label: Label) -> Result<usize, std::io::Error> {
         self.label_slots.push(label);
         if self.label_slots.capacity() > 1472 {
             Err(Error::new(ErrorKind::Other, "No slots left"))
@@ -439,10 +465,9 @@ impl LabelStorageArea {
     /// # Ok::<(), std::io::Error>(())
     /// ```
     pub fn push_region_label(&mut self, size: usize) -> Result<usize, std::io::Error> {
-        self.push_label(REGION(RegionLabel::new(
-            size,
-            self.label_slots.len() as u32,
-        )))
+        let mut region = RegionLabel::new().size(size);
+        region.update_checksum();
+        self.push_label(Region(region))
     }
 
     /// Create a new namespace label in the next available slot. Returns the slot number.
@@ -467,10 +492,9 @@ impl LabelStorageArea {
     /// # Ok::<(), std::io::Error>(())
     /// ```
     pub fn push_namespace_label(&mut self, size: usize) -> Result<usize, std::io::Error> {
-        self.push_label(NAMESPACE(namespace_label::NamespaceLabel::new(
-            size,
-            self.label_slots.len() as u32,
-        )))
+        let mut namespace = NamespaceLabel::new().size(size);
+        namespace.update_checksum();
+        self.push_label(Namespace(namespace))
     }
 
     /// Create an empty label and returns the slot number.
@@ -479,7 +503,7 @@ impl LabelStorageArea {
     /// [push_namespace_label](#method.push_namespace_label). The function isn't very useful and
     /// will probably be deprecated in the future.
     pub fn push_empty_label(&mut self) -> Result<usize, std::io::Error> {
-        self.push_label(EMPTY)
+        self.push_label(Empty)
     }
 
     /// Replaces a given slot with a label.
@@ -549,12 +573,12 @@ impl LabelStorageArea {
         for label in &self.label_slots {
             // FIXME: can we make this not so ugly?
             let l = match label {
-                REGION(r) => r.as_bytes(),
-                NAMESPACE(n) => n.as_bytes(),
-                VENDOR(_v) => &[0; 256], //FIXME
+                Region(r) => r.as_bytes(),
+                Namespace(n) => n.as_bytes(),
+                Vendor(_v) => &[0; 256], //FIXME
                 _ => &[0; 256],
             };
-            n = n + match v.write(l) {
+            n += match v.write(l) {
                 Err(e) => return Err(e),
                 Ok(_) => 256,
             }
@@ -587,9 +611,8 @@ impl LabelStorageArea {
         let active = self.active_lib as usize;
         let index = &self.label_index_blocks[active];
 
-        match index.valid() {
-            Err(e) => return Err(Error::new(ErrorKind::Other, e)),
-            Ok(_) => {}
+        if let Err(e) = index.valid() {
+            return Err(Error::new(ErrorKind::Other, e));
         }
 
         // TODO: Call verify on all active slots
@@ -610,10 +633,10 @@ impl Serialize for LabelStorageArea {
         state.serialize_field("LBI1", &self.label_index_blocks[1])?;
         for (_i, label) in self.label_slots.iter().enumerate() {
             match label {
-                Label::REGION(region) => state.serialize_field("region", region),
-                Label::NAMESPACE(namespace) => state.serialize_field("namespace", namespace),
-                Label::VENDOR(raw) => state.serialize_field("vendor", raw),
-                Label::EMPTY => state.serialize_field("empty", &0),
+                Label::Region(region) => state.serialize_field("region", region),
+                Label::Namespace(namespace) => state.serialize_field("namespace", namespace),
+                Label::Vendor(raw) => state.serialize_field("vendor", raw),
+                Label::Empty => state.serialize_field("empty", &0),
             }?;
         }
         state.end()
